@@ -8,14 +8,15 @@ package PGObject;
 use strict;
 use warnings;
 use Carp;
+use Memoize;
 
 =head1 VERSION
 
-Version 1.4.1
+Version 1.402.6
 
 =cut
 
-our $VERSION = '1.400.1';
+our $VERSION = '1.402.6';
 
 my %typeregistry = (
     default => {},
@@ -23,6 +24,13 @@ my %typeregistry = (
 
 =head1 SYNPOSIS
 
+To use without caching:
+
+  use PGObject;
+
+To use with caching:
+
+  use PGObject ':cache'; 
 
 To get basic info from a function
 
@@ -63,6 +71,13 @@ To do the same with a running total
       running_funcs => [{agg => 'sum(amount)', alias => 'running_total'}],
   );
 
+=cut
+
+sub import {
+    my @directives = @_;
+    memoize 'function_info' if grep { $_ eq ':cache' } @directives;
+}
+
 =head1 DESCRIPTION
 
 PGObject contains the base routines for object management using discoverable
@@ -82,6 +97,18 @@ idea that wrapper classes would be written to implement this.
 
 =head1 FUNCTIONS
 
+=head2 clear_info_cache
+
+This function clears the info cache if this was loaded with caching enabled.
+
+The cache is also automatically cleared when a function that was run could not
+be found (this could be caused by updating the db).
+
+=cut
+
+sub clear_info_cache {
+    eval { Memoize::flush_cache('function_info') };
+}
 
 
 =head2 function_info(%args)
@@ -178,6 +205,7 @@ sub function_info {
     my $sth = $dbh->prepare($query) || die $!;
     $sth->execute(@queryargs);
     my $ref = $sth->fetchrow_hashref('NAME_lc');
+    croak "transaction already aborted" if  $dbh->state eq '25P02';
     croak "No such function" if !$ref;
     croak 'Ambiguous discovery criteria' if $sth->fetchrow_hashref('NAME_lc');
 
@@ -255,55 +283,38 @@ sub call_procedure {
 
     my $registry = $typeregistry{$args{registry}};
     my $dbh = $args{dbh};
+    croak "No database handle provided" unless $dbh;
+    croak "dbh not a database handle" unless eval {$dbh->isa('DBI::db')};
 
     my $wf_string = '';
 
-    if ($args{running_funcs}){
-        for (@{$args{running_funcs}}){
-           $wf_string .= ', '. $_->{agg}. ' OVER (ROWS UNBOUNDED PRECEDING) AS '
-                         . $_->{alias};
-        }
-    }
-    my @qargs = ();
-    my $argstr = '';
-    for my $in_arg (@{$args{args}}){
-        my $arg = $in_arg;
-        if (eval {$in_arg->can('pgobject_to_db')}) {
-            $arg = $in_arg->{pgobject_to_db};
-        } 
-            
-        if ($argstr){
-           $argstr .= ', ?';
-        } else {
-           $argstr .= '?';
-        }
-        if (ref $arg eq ref {}){
-           $argstr .= "::".$dbh->quote_identifier($arg->{cast}) if $arg->{cast};
-           push @qargs, $arg->{value};
-        }  else {
-           push @qargs, $arg;
-        }
-    }
+    $wf_string = join ', ', map { 
+                         $_->{agg} 
+                         . ' OVER (ROWS UNBOUNDED PRECEDING) AS '
+                         . $_->{alias}
+                 } @{$args{running_funcs}} if $args{running_funcs};
+    $wf_string = ', ' . $wf_string if $wf_string;
+
+    my @qargs = map { 
+                      my $arg = $_;
+                      $arg = $arg->to_db if eval {$arg->can('to_db')};
+                      $arg = $arg->pgobject_to_db if eval {$arg->can('pgobject_to_db')};
+                      ref $arg ? $arg->{value} : $arg 
+                }  @{$args{args}};
+
+    my $argstr = join ', ', map { 
+                  (ref $_ and $_->{cast}) ? "?::$_->{cast}" : '?';
+                  } @{$args{args}};  
+
     my $order = '';
     if ($args{orderby}){
-        for my $ord (@{$args{orderby}}){
-            my @words = split / /, $ord;
-            my $direction = pop @words;
-            my $safe_ord;
-
-            if (uc($direction) =~ /^(ASC|DESC)$/){
-              $ord =~ s/\s+$direction$//;
-              $safe_ord = $dbh->quote_identifier($ord) . " $direction"; 
-            } else {
-               $safe_ord = $dbh->quote_identifier($ord);
-            }
- 
-            if ($order){
-                $order .= ', ' . $safe_ord;
-            } else {
-                $order =  $safe_ord;
-            }
-        }
+        $order = join(', ', map {
+                                  $_ =~ s/\s+(ASC|DESC)$//i;
+                                  my $dir = $1;
+                                  defined $dir ? $dbh->quote_identifier($_)
+                                                  . " $dir"
+                                               : $dbh->quote_identifier($_);
+                                } @{$args{orderby}}); 
     }
     my $query = qq|
            SELECT * $wf_string 
@@ -343,6 +354,8 @@ sub call_procedure {
     }
 
     $sth->execute();
+    
+    clear_info_cache() if $dbh->state eq '42883'; # (No Such Function)
 
     my @rows = ();
     while (my $row = $sth->fetchrow_hashref('NAME_lc')){
@@ -371,7 +384,7 @@ This module should generally only be used by type handlers or by this module.
 =cut
 
 sub process_type {
-    my ($val, $type, $registry) = @_;
+    my ($val, $type, $registry, $dbh) = @_;
 
     $registry = $typeregistry{$registry} unless ref $registry;
     # Array handling as we'd get this usually from DBD::Pg or equivalent
@@ -487,9 +500,9 @@ sub get_type_registry {
 
 One of the powerful features of PGObject is the ability to declare methods in
 types which can be dynamically detected and used to serialize data for query
-purposes. Objects which contain a pgobject_to_db(), that method will be called
-and the return value used in place of the object.  This can allow arbitrary
-types to serialize themselves in arbitrary ways.
+purposes. Objects which contain a pgobject_to_db() or a to_db() method, that 
+method will be called and the return value used in place of the object.  This 
+can allow arbitrary types to serialize themselves in arbitrary ways.
 
 For example a date object could be set up with such a method which would export
 a string in yyyy-mm-dd format.  An object could look up its own definition and
@@ -558,6 +571,22 @@ We do not supply type information, If your top-level module needs this, please
 check out https://code.google.com/p/typeutils/ which could then be used via our
 function mapping APIs here.
 
+=head2 Safely Handling Memoization of Catalog Lookups
+
+It is important to remember, when writing PGObject top half frameworks that the
+catalog lookups may be memoized and may come back as a data structure.  This
+means that changes to the structures returned from get_function_info() in this
+module and similar functions in other catalog-bound modules may not be safe to
+modify in arbitrary ways.  Therefore we recommend that the return values from
+catalog-lookup functions are treated as immutable.
+
+Normalizing output is safe provided there are no conflicts between naming 
+conventions.  This is usually true since different naming conventions would 
+interfere withmapping.  However, there could be cases where it is not true, for
+example, where two different mapping modules agree on a subset of normalization
+conventions but differ on some details.  The two might safely handle the same
+conventions but normalize differently resulting in conflicts of both were used.
+
 =head1 A BRIEF GUIDE TO THE NAMESPACE LAYOUT
 
 Most names underneath PGObject can be assumed to be top-half modules and modules
@@ -643,7 +672,7 @@ not be half of what it is today.
 
 =head1 COPYRIGHT
 
-COPYRIGHT (C) 2013 Chris Travers
+COPYRIGHT (C) 2013-2014 Chris Travers
 
 Redistribution and use in source and compiled forms with or without 
 modification, are permitted provided that the following conditions are met:
